@@ -7,6 +7,7 @@
 # 4) getHFDitemavail()
 # 5) HFCparse()
 # 6) getHFCcountries()
+# 7) extract_HFD_items()
 
 ############################################################################
 # 1) HFDparse()
@@ -61,14 +62,20 @@ HFDparse <- function(DF){
 #' @return a vector of HFD country short codes.
 #' 
 #' @importFrom XML readHTMLTable
+#' @importFrom httr GET
 #' 
 #' @export
 #' 
 getHFDcountries <- function(){
 	# the ugliest code I've ever written. There must be a better way..
-	X <- XML::readHTMLTable("http://www.humanfertility.org/cgi-bin/zipfiles.php",header=TRUE,
-			colClasses=c("character","character"),which=2,stringsAsFactors = FALSE)
-	gsub("\\s*\\([^\\)]+\\)","",X[,2])
+  hfd_url <- "https://www.humanfertility.org/cgi-bin/zipfiles.php"
+  some_html <- httr::GET(hfd_url)
+  X <- XML::readHTMLTable(XML::htmlParse(some_html),
+                          header = TRUE, 
+                          colClasses = c("character", "character"), 
+                          which = 2, 
+                          stringsAsFactors = FALSE)
+	gsub("\\s*\\([^\\)]+\\)", "", X[,2])
 }
 
 
@@ -84,17 +91,24 @@ getHFDcountries <- function(){
 #' 
 #' @return character string of eight integers representing the date as \code{"yyyymmdd"}.
 #' 
-#' @importFrom RCurl getURL
+#' @importFrom httr GET content
 #' 
 #' @export
 #' 
 getHFDdate <- function(CNTRY){
-	CountryURL      <- paste0("http://www.humanfertility.org/cgi-bin/country.php?country=",CNTRY)
-# date of last update will be the most common component, it turns out, since it forms part of the links:
-	partssummary    <- table(unlist(strsplit(RCurl::getURL(CountryURL), "\\\\")))
-	LastUpdate      <- names(partssummary)[which.max(partssummary)]
-	# this isn't a date string, just 8 digits squashed together yyyymmdd
-	LastUpdate
+  CountryURL      <- paste0("http://www.humanfertility.org/cgi-bin/country.php?",
+                            "country=", CNTRY)
+  #get date string by finding the permalink listed at the bottom of page.
+  html_country    <- httr::GET(CountryURL)
+  htmlstr <- httr::content(html_country, as="text")
+  LastUpdate <- regmatches(htmlstr, regexpr("(?<=update=)20[0-9]{6}", htmlstr,
+                                            perl=T))
+  if(length(LastUpdate)==0){
+    stop("I can't find the date of the latest update to the data for this
+          country. The Human Fertility Database website may have changed")
+  }
+  # this isn't a date string, just 8 digits squashed together yyyymmdd
+  LastUpdate
 }
 
 
@@ -108,26 +122,32 @@ getHFDdate <- function(CNTRY){
 #' 
 #' @param CNTRY HFD country short code.
 #' 
-#' @return a vector of item names. These are the file base names, and only need the extension \code{.txt} added in order to get te file name.
+#' @return a vector of item names. These are the file base names, and only need the extension \code{.txt} added in order to get the file name.
 #' 
-#' @importFrom RCurl getURL
+#' @importFrom httr GET content
 #' 
 #' @export
 #' 
 getHFDitemavail <- function(CNTRY){
-	# again, very ugly parsing, there must be a better way
-	CountryURL      <- paste0("http://www.humanfertility.org/cgi-bin/country.php?country=",CNTRY)
-# date of last update will be the most common component, it turns out, since it forms part of the links:
-	partssummary    <- table(unlist(strsplit(RCurl::getURL(CountryURL), "\\\\")))
-	partsasfr       <- table(unlist(strsplit(RCurl::getURL(paste0(CountryURL,"&tab=asfr&t1=3&t2=4")), "\\\\")))
-	partsft         <- table(unlist(strsplit(RCurl::getURL(paste0(CountryURL,"&tab=ft&t1=5&t2=6")), "\\\\")))
-	parts           <- c(names(partssummary), names(partsasfr),names(partsft))
+  # It seems this function will only worked if you are logged in
+	CountryURL      <- paste0("https://www.humanfertility.org/cgi-bin/",
+	                          "country.php?country=",CNTRY)
+
+	# vector of names of tabs on each HFD page
+	tabs <- c("si", "asfr", "ft")
+
+	item_list <- lapply(tabs, function(tab){
+	  # iterate through each tab and construct URL
+	  url <- paste0(CountryURL, "&tab=", tab)
+	  tab_html  <- httr::content(httr::GET(url))
+	  return(extract_HFD_items(tab_html, CNTRY))
+	})
 	
-	items           <- unlist(lapply(strsplit(parts, split = "&"),"[[",1))
-	items           <- gsub(CNTRY,"",gsub("\\.txt","",items[grepl(items, pattern = ".txt")]))
-	items
+	return(unlist(item_list))
 }
 
+
+
 ############################################################################
 # 5) HFCparse()
 ############################################################################
@@ -144,8 +164,7 @@ getHFDitemavail <- function(CNTRY){
 #' @details This parse routine is based on the subjective opinions of the author...
 #' 
 #' @export
-#' 
-
+#'
 HFCparse <- function(DF){
 	# get just one year, and treat it like age groups, where we
 	# mark the lower bound and provide a year interval column
@@ -198,8 +217,35 @@ getHFCcountries <- function(names = FALSE){
 
 
 
-
-
+############################################################################
+# 7) extract_HFD_items()
+############################################################################
+#' @title Extract names of HFD items from HTML
+#'
+#' @description Extract names of Human fertility database datasets from an
+#' HTML page. Links are extracted from html, and for each link, any text falling
+#' between the provided country code and a .txt extension is returned
+#'
+#' @param html_content An object of class \code{xml_document} taken from a
+#' country page of the HFD website, typically the result of
+#' \code{httr::content(httr::GET(url))}
+#'
+#' @param CNTRY A string given one of the country codes used by HFD. See
+#' \code{getHFDcountries} for a function which lists available countries.
+#'
+#' @return A vector of characters, each of which is an item available for
+#' download
+#'
+#' @importFrom XML getHTMLLinks htmlParse
+#'
+extract_HFD_items <- function(html_content,CNTRY) {
+  parsed_html <- XML::htmlParse(html_content)
+  all_links <- XML::getHTMLLinks(parsed_html)
+  item_regex <- regexpr(paste0("(?<=",CNTRY,")[A-Za-z0-9]+(?=\\.txt)"), 
+          all_links, perl=T)
+  item_names <- regmatches(all_links, item_regex)
+  return(item_names)
+}
 
 
 
